import {observer} from "mobx-react-lite";
import React, {SyntheticEvent, useCallback, useEffect, useMemo, useState} from "react";
import {InputField, InputFieldProps, useModelContext} from "@rx-suite/ui";
import {ValidationError} from "yup";
import { ParameterToSchemaConversion } from "../../pages/alert_rules/form/tabs/AlertRuleTriggerTab";
import {AlertRuleModel} from "../../stores/AlertRuleStore";

type ParameterInputProps = {
    ruleModel: "MachineRemainsInStateRuleParameters";
    name: string;
    value?: string | number;
    onChange?: (event: SyntheticEvent) => void
} & Omit<InputFieldProps, 'onChange' | 'value'>

export const ParameterInput: React.FunctionComponent<ParameterInputProps> = observer(({
                                                                                          ruleModel,
                                                                                          name,
                                                                                          value,
                                                                                          onChange,
                                                                                          ...props
}) => {
    const label = ParameterToSchemaConversion[ruleModel][name].label;
    const validation = ParameterToSchemaConversion[ruleModel][name].validation;

    const model = useModelContext() as AlertRuleModel;
    const [error, setError] = useState<string>('')

    const validate = () => {
        try {
            validation.validateSync(model.viewModel.parameters[name]);
            setError('');
        } catch (e) {
            if(!(e instanceof ValidationError)) {throw e}
            setError(e.message);
        }
    }

    useEffect(() => {
        validate();
    }, [])
    
    const handleChange = useCallback((event: SyntheticEvent) => {
        if(onChange) {
            onChange(event);
        } else  {
            model.viewModel.parameters[name] = event.target.value;
        }
        validate();
    }, [model.viewModel.parameters, name, onChange, validate])

    const resolvedValue = useMemo(() => {
        if(value) {return value;}
        return model.viewModel.parameters[name];
    }, [model.viewModel.parameters, name, value])
    
    return <InputField label={label}
                       onChange={handleChange}
                       value={resolvedValue}
                       error={error !== ''}
                       helperText={error}
                       {...props}/>
})