import {attribute, DataStore, Model, model, Types} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {AlertShiftModel} from "./AlertShiftStore";
import {action} from "mobx";

export type AlertType = "MachineInState" | "MachineTimeDeviation"

type EmailDefaultData = {
    subject: string;
    body: string;
}

const ALERT_EMAIL_DEFAULT_TEXT: {[key: string]: EmailDefaultData} = {
    "MachineInState" : {
        subject: "MES360 ALERT: [machine]([sn]) is in [state] since [time]!",
        body: "This is an automated email generated by MES360 to alert you that the machine [machine]([sn]) is in the [state] state since [time]."
    },
    "MachineTimeDeviation" : {
        subject: "MES360 ALERT: [machine]([sn]) time synchronization issue!",
        body: "This is an automated email generated by MES360 to alert you that the machine [machine]([sn]) time settings are no longer synchronized with the application server time. Please verify machine settings."
    }
}

@model({
    transportLayer: "/api/alert_rules",
    permissionTarget: "AlertRule",
    routingInfo: {
        create: '/settings/alert_rules/create',
        read: '/settings/alert_rules/:id/edit',
        update: '/settings/alert_rules/:id/edit'
    }
})

export class AlertRuleModel extends Model {
    types: Types =  {
        parameters: 'json'
    }

    @attribute({
        label: t`Alert Name`,
        validation: yup.string().required(t`Alert Name is a required field`)
    })
    name = '';

    @attribute({
        label: t`Alert Description`,
        validation: yup.string()
    })
    description = '';

    @attribute({
        label: t`Enabled`,
        validation: yup.boolean()
    })
    enabled = true;

    @attribute({
        label: t`Alert Email Subject`,
        validation: yup.string().required(t`Alert Email Subject is a required field.`).max(250)
    })
    email_subject: string = ALERT_EMAIL_DEFAULT_TEXT['MachineInState'].subject

    @attribute({
        label: t`Alert Email Body`,
        validation: yup.string().required(t`Alert Email Body is a required field.`)
    })
    email_body: string = ALERT_EMAIL_DEFAULT_TEXT['MachineInState'].body

    @attribute({
        label: t`Machines or Machine Groups to Monitor`,
        validation: yup.array().min(1).required(t`At least one machine or machine group must be selected.`)
    })
    alert_targets: string[] = [];

    @attribute({})
    parameters: Record<string, unknown> = {};

    @attribute({
        label: t`Alert Trigger`,
        validation: yup.string()
    })
    type: AlertType = "MachineInState";

    @attribute({})
    alert_rule_id: string = ''

    @attribute({})
    alert_recipients_to_add: AlertShiftModel[] = [];

    @action
    changeAlertType(type: AlertType): void {
        this.viewModel.type = type;
        this.viewModel.parameters = {};
        this.viewModel.email_subject = ALERT_EMAIL_DEFAULT_TEXT[type].subject;
        this.viewModel.email_body = ALERT_EMAIL_DEFAULT_TEXT[type].body;
    }

    constructor() {
        super();
        super.setup();
    }
}



export const AlertRuleStore = AlertRuleModel.store as DataStore<AlertRuleModel>;
