class AlertRule < ApplicationRecord
  # Base Model for all Alert Rules
  has_many :alert_shifts, dependent: :destroy, autosave: true
  has_many :alert_rule_targets, autosave: true, dependent: :destroy

  class << self
    def type_to_class
      {
        MachineInState: MachineInState,
        MachineTimeDeviation: MachineTimeDeviation
      }
    end

    def new_alert_model(type)
      c = type_to_class[type.to_sym]
      c.new
    end

    def execute_all
      return if AlertRule.count.zero? || ExternalDataCache::RxHub.instance.shifts.empty?

      AlertRule.where(enabled: true).find_each do |rule|
        rule.execute
      rescue => e
        Rails.logger.error "Failed to execute alert with id #{rule.id}. Error: #{e}"
      end
    end

    def replace_name_with_alert_name(action)
      if action[:id] == "name"
        action[:id] = "alert_rule.name"
      end
      action
    end
  end

  def set_data(params)
    assign_attributes({
                                   name: params[:name],
                                   type: params[:type],
                                   enabled: params[:enabled],
                                   description: params[:description],
                                   email_subject: params[:email_subject],
                                   email_body: params[:email_body],
                                   parameters: params[:parameters]
                                 })
  end

  def update_targets(targets)
    remove_deleted_targets(targets) unless new_record?
    targets.each do |target_id|
      type = AlertRuleTarget.resolve_target_type(target_id.split('-')[0])
      exists = false
      exists = alert_rule_targets.find_by(target_id: target_id) unless new_record?
      alert_rule_targets.build(alert_target_type: type, target_id: target_id) unless exists
    end
  end

  def send_alert(target_id, identifiers, email_params)
    identifiers.each do |identifier|
      rule_target = alert_rule_targets.find_by(target_id: identifier)
      # Check if there's recipients for this shift
      if rule_target && is_active_with_current_shifts?
        emit(target_id, build_email_data(email_params), build_recipient_list)
        break # no need to continue checking alerts for this target
      end
    end
  end

  def resolved_parameters
    JSON.parse(parameters).symbolize_keys
  end

  def to_json(*_args)
    render_jbuilder_partial
  end

  def render_jbuilder_partial
    AlertRulesController.render(
      partial: to_partial_path,
      locals: {broadcast_local_name => self},
      formats: [:json],
      handlers: [:jbuilder]
    )
  end

  def execute
    # each alert rules should implement it's own execute method
    raise NotImplementedError
  end

  def add_shifts(recipient_data)
    recipient_data.each do |hash|
      shift_id = hash[:shift_id]
      recipient_ids = hash[:alert_recipients]
      shift = AlertShift.new(shift_id: shift_id)
      shift.alert_recipients.build(recipient_ids.map {|id| {target_id: id}})
      # add_to_target adds to the collection without saving.
      association(:alert_shifts).add_to_target(shift)
    end
  end

  def build_recipient_list
    shifts = ExternalDataCache::RxHub.instance.shifts
    emails = []
    shift_ids = shifts.map {|s| s[:shift][:id]}
    alert_shift_models = alert_shifts.where(shift_id: shift_ids)
    alert_shift_models.map do |alert_shift_model|
      alert_shift_model.alert_recipients.each do |alert_recipient|
        email = alert_recipient.resolve_email_address
        emails.push(email) unless email.nil?
      end
    end
    emails.uniq
  end

  def is_active_with_current_shifts?
    shifts = ExternalDataCache::RxHub.instance.shifts
    alert_shifts.exists?(shift_id: shifts.map {|shift| shift[:shift][:id]})
  end

  private

  def emit(alert_target, alert_data, mail_to)
    return if Alert.target_has_active?(alert_target, id)

    alert = Alert.new(
      target_identifier: alert_target,
      alert_subject: apply_data(email_subject, alert_data),
      alert_body: apply_data(email_body, alert_data),
      sent_to: mail_to.join(",")
    )

    alert.alert_rule = self
    alert.save!

    unless Alert.local_only? || mail_to.count.zero?
      Rails.logger.info "SENDING ALERT #{alert.to_json} to #{mail_to}"
      RxModule::Communicator::Emails.send_email(mail_to, alert.alert_subject, alert.alert_body)
    end
  rescue StandardError => e
    Rails.logger.warn "Failed to send alert for machine #{alert_data[:name]} (#{alert_data[:sn]}) : #{e}"
  end

  def target_id(machine_id)
    "m-#{machine_id}"
  end

  def build_email_data(email_params)
    raise NotImplementedError
  end

  def apply_data(content, data)
    content_copy = content.dup

    data.each do |replace, value|
      content_copy.gsub!(replace, value)
    end
    content_copy
  end

  def remove_deleted_targets(new_targets)
    alert_rule_targets.each do |t|
      keep = new_targets.detect { |n| t.target_id == n }
      t.mark_for_destruction unless keep
    end
  end
end
